@@ -1,7 +1,7 @@
 ;;; cal-mayan.el --- calendar functions for the Mayan calendars
 
 ;; Copyright (C) 1992, 1993, 1995, 1997, 2001, 2002, 2003, 2004, 2005,
-;;   2006  Free Software Foundation, Inc.
+;;   2006, 2007  Free Software Foundation, Inc.
 
 ;; Author: Stewart M. Clamen <clamen@cs.cmu.edu>
 ;;	Edward M. Reingold <reingold@cs.uiuc.edu>
@@ -26,7 +26,7 @@
 ;; Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 ;; Boston, MA 02110-1301, USA.
 
-;;; Synched up with: FSF Emacs 22.1 CVS 2006-09-15
+;;; Synched up with: FSF Emacs 22 CVS 2007-03-30
 
 ;;; Commentary:
 
@@ -262,8 +262,7 @@ Returns nil if such a tzolkin-haab combination is impossible."
          (haab-month-list (append calendar-mayan-haab-month-name-array
                                   (and (< haab-day 5) '("Uayeb"))))
          (haab-month (cdr
-                      ;; XEmacs change, we don't have assoc-string
-                      (cal-assoc-string
+                      (assoc-string
                        (completing-read "Haab uinal: "
                                         (mapcar 'list haab-month-list)
                                         nil t)
@@ -278,8 +277,7 @@ Returns nil if such a tzolkin-haab combination is impossible."
                          '(lambda (x) (and (> x 0) (< x 14)))))
          (tzolkin-name-list (append calendar-mayan-tzolkin-names-array nil))
          (tzolkin-name (cdr
-                        ;; XEmacs change, we don't have assoc-string
-                        (cal-assoc-string
+                        (assoc-string
                           (completing-read "Tzolkin uinal: "
                                            (mapcar 'list tzolkin-name-list)
                                            nil t)
